using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Data.SQLite;
//#Include:System.Data.dll
//#Include:System.Data.SQLite.dll

namespace PandoraBox
{
    public class CBARTICLEMixPlugin : IPluginMacro
    {
        Regex _block = new Regex(@"\[(GOOGLE|BING|YANDEX)ARTICLEMIX-(\d)-(\d)(?<mp>#\d+)*\]", RegexOptions.Compiled);
        Regex _customBlock = new Regex(@"\[MYCB\(([^)]+)\)ARTICLEMIX-(\d)-(\d)(?<mp>#\d+)*\]", RegexOptions.Compiled);
        int _minSentances = 20;

        public string Execute(string template, PluginMacroArgs args)
        {
            template = Load(template, _block, args);
            template = Load(template, _customBlock, args);
            return template;
        }

        private string Load(string template, Regex regex, PluginMacroArgs args)
        {
            return regex.Replace(template, delegate (Match match)
            {
                var type = match.Groups[1].Value + ".Directory";
                String typePath = GetIniParameter(type, args);
                bool dbMode = GetIniParameter("Main.UseDatabase", args) == "1";
                String key = CleanKey(GetKey(args, match));
                int takeArticles = Rnd.Next(int.Parse(match.Groups[2].Value), int.Parse(match.Groups[3].Value));
                string article = dbMode
                    ? GetDBArticleMix(args, GetIniParameter("Main.DataFolder", args), typePath.Replace(".", "_"), key, takeArticles)
                    : GetFileArticleMix(Path.Combine(GetIniParameter("Main.DataFolder", args), typePath, GetSub(key.Trim()), key), takeArticles);
                if (article == null) //статей по требованиям нет, выводим стандартным макросом текста
                    return "[FREETEXT-3-5/p]";
                article = "{PARAGRAPH}" + article + "{/PARAGRAPH}"; //добавляем параграфы
                return article;
            });
        }

        private string GetIniParameter(String name, PluginMacroArgs args)
        {
            String root = args.Context[name] as String;
            if (root == null)
            {
                root = File.ReadAllLines("ContentBox.ini")
                    .FirstOrDefault(x => x.StartsWith(name + "=", StringComparison.InvariantCultureIgnoreCase));
                root = root != null ? root.Split('=')[1] : "";
                args.Context[name] = root;
            }
            return root;
        }

        private string GetSub(string key)
        {
            return key.Length <= 2 ? key : key.Substring(0, 2).Trim();
        }

        private string GetFileArticleMix(String file, int takeArticles)
        {
            if (!File.Exists(file)) return null;
            var lines = Shuffle(File.ReadAllLines(file).ToList());
            return MixArticles(lines, takeArticles);
        }

        private string MixArticles(List<String> lines, int takeArticles)
        {
            List<String[]> sentancesArray = new List<String[]>();
            for (int i = 0; i < lines.Count; i++)
            {
                var sentances = lines[i].Split('|');
                if (sentances.Length >= _minSentances)
                {
                    sentancesArray.Add(sentances);
                    if (sentancesArray.Count >= takeArticles)
                        break;
                }
            }
            if (sentancesArray.Count >= 2)
                return String.Join(" ", sentancesArray.SelectMany(x => RandomPart(x, sentancesArray.Count)));
            return null;
        }

        private List<String> RandomPart(string[] sentances, int partsCount)
        {
            if (partsCount < 2) return sentances.ToList();
            var length = sentances.Length / partsCount;
            if (length == 0) return new List<string>();
            var offset = Rnd.Next(0, sentances.Length - length);
            if (offset < 0) offset = 0;
            return sentances.Skip(offset).Take(length).ToList();
        }

        private string GetKeysFile(PluginMacroArgs args)
        {
            return args.Context["$dbname"] as String ?? "content.db";
        }

        private string GetDBArticleMix(PluginMacroArgs args, String dbDir, String table, String key, int takeArticles)
        {
            //connect
            var contextKey = "SQLiteConnect";
            SQLiteConnection connection = args.Context[contextKey] as SQLiteConnection;
            if (connection == null)
            {
                connection = new SQLiteConnection("Data Source = \"" + dbDir + "/" + GetKeysFile(args) + "\"; Journal Mode=Off;");
                connection.Open();
                args.Context[contextKey] = connection;
            }

            //reader command
            contextKey = "SQLiteCommand" + table;
            SQLiteCommand reader = args.Context[contextKey] as SQLiteCommand;
            if (reader == null)
            {
                reader = new SQLiteCommand("select content from '" + table + "' where key = :key", connection);
                reader.Parameters.Add(new SQLiteParameter("key", ""));
                args.Context[contextKey] = reader;
                if (!TableExists(table, connection)) reader.CommandText = null;
            }
            if (reader.CommandText == null) return null; //no table

            reader.Parameters[0].Value = key;
            var content = (reader.ExecuteScalar() as string) ?? "";
            var lines = Shuffle(content.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries).ToList());
            return MixArticles(lines, takeArticles);
        }

        private bool TableExists(String table, SQLiteConnection connection)
        {
            return new SQLiteCommand("SELECT count(*) FROM sqlite_master WHERE name='" + table + "'", connection).ExecuteScalar().ToString() == "1";
        }

        private string GetKey(PluginMacroArgs args, Match match)
        {
            int keyIndex;
            string pos = null;
            var result = args.Key;
            if (args.KeyParts != null && (pos = match.Groups["mp"].Value) != "" && (keyIndex = int.Parse(pos.Trim('#'))) > 0)
                result = args.KeyParts[keyIndex > args.KeyParts.Length ? 0 : keyIndex - 1];
            return result;
        }

        private string CleanKey(String key)
        {
            String key1 = key;
            key1 = String.Join("", key1.Split(Path.GetInvalidPathChars()));
            key1 = String.Join("", key1.Split(Path.GetInvalidFileNameChars()));
            return key1;
        }

        private static List<T> Shuffle<T>(List<T> items)
        {
            if (items.Count < 2) return items;

            for (int i = items.Count - 1; i > 0; i--)
            {
                int j = Rnd.InnerObject.Next(i); //Фишер–Йетс
                T temp = items[i];
                items[i] = items[j];
                items[j] = temp;
            }
            return items;
        }

        public ushort Level { get { return 5; } }
        public String Name { get { return "CBARTICLEMixPlugin"; } }
    }
}